#
# Copyright (C) 2011 OpenWrt.org
#

USE_REFRESH=1

. /lib/ipq806x.sh
. /lib/upgrade/common.sh

RAMFS_COPY_DATA=/lib/ipq806x.sh
RAMFS_COPY_BIN="/usr/bin/dumpimage /bin/mktemp /usr/sbin/mkfs.ubifs
	/usr/sbin/ubiattach /usr/sbin/ubidetach /usr/sbin/ubiformat /usr/sbin/ubimkvol
	/usr/sbin/ubiupdatevol /usr/bin/basename /bin/rm /usr/bin/find
	/usr/sbin/mkfs.ext4"

get_full_section_name() {
	local img=$1
	local sec=$2

	dumpimage -l ${img} | grep "^ Image.*(${sec})" | \
		sed 's,^ Image.*(\(.*\)),\1,'
}

image_contains() {
	local img=$1
	local sec=$2
	dumpimage -l ${img} | grep -q "^ Image.*(${sec}.*)" || return 1
}

print_sections() {
	local img=$1

	dumpimage -l ${img} | awk '/^ Image.*(.*)/ { print gensub(/Image .* \((.*)\)/,"\\1", $0) }'
}

image_has_mandatory_section() {
	local img=$1
	local mandatory_sections=$2

	for sec in ${mandatory_sections}; do
		image_contains $img ${sec} || {\
			return 1
		}
	done
}

image_demux() {
	local img=$1

	for sec in $(print_sections ${img}); do
		local fullname=$(get_full_section_name ${img} ${sec})

		local position=$(dumpimage -l ${img} | grep "(${fullname})" | awk '{print $2}')
		dumpimage -i ${img} -o /tmp/${fullname}.bin -T "flat_dt" -p "${position}" ${fullname} > /dev/null || { \
			echo "Error while extracting \"${sec}\" from ${img}"
			return 1
		}
	done
	return 0
}

image_is_FIT() {
	if ! dumpimage -l $1 > /dev/null 2>&1; then
		echo "$1 is not a valid FIT image"
		return 1
	fi
	return 0
}

switch_layout() {
	local layout=$1
	local boot_layout=`find / -name boot_layout`

	# Layout switching is only required as the  boot images (up to u-boot)
	# use 512 user data bytes per code word, whereas Linux uses 516 bytes.
	# It's only applicable for NAND flash. So let's return if we don't have
	# one.

	[ -n "$boot_layout" ] || return

	case "${layout}" in
		boot|1) echo 1 > $boot_layout;;
		linux|0) echo 0 > $boot_layout;;
		*) echo "Unknown layout \"${layout}\"";;
	esac
}

do_flash_mtd() {
	local bin=$1
	local mtdname=$2
	local append=""

	local mtdpart=$(grep "\"${mtdname}\"" /proc/mtd | awk -F: '{print $1}')
	local pgsz=$(cat /sys/class/mtd/${mtdpart}/writesize)
	[ -f "$CONF_TAR" -a "$SAVE_CONFIG" -eq 1 -a "$2" == "rootfs" ] && append="-j $CONF_TAR"

	dd if=/tmp/${bin}.bin bs=${pgsz} conv=sync | mtd $append -e "/dev/${mtdpart}" write - "/dev/${mtdpart}"
}

do_flash_emmc() {
	local bin=$1
	local emmcblock=$2

	dd if=/dev/zero of=${emmcblock}
	dd if=/tmp/${bin}.bin of=${emmcblock}
}

do_flash_partition() {
	local bin=$1
	local mtdname=$2
	local emmcblock="$(find_mmc_part "$mtdname")"

	if [ -e "$emmcblock" ]; then
		do_flash_emmc $bin $emmcblock
	else
		do_flash_mtd $bin $mtdname
	fi
}

do_flash_bootconfig() {
	local bin=$1
	local mtdname=$2

	# Fail safe upgrade
	if [ -f /proc/boot_info/getbinary_${bin} ]; then
		cat /proc/boot_info/getbinary_${bin} > /tmp/${bin}.bin
		do_flash_partition $bin $mtdname
	fi
}

do_flash_failsafe_partition() {
	local bin=$1
	local mtdname=$2
	local emmcblock
	local primaryboot

	# Fail safe upgrade
	[ -f /proc/boot_info/$mtdname/upgradepartition ] && {
		default_mtd=$mtdname
		mtdname=$(cat /proc/boot_info/$mtdname/upgradepartition)
		primaryboot=$(cat /proc/boot_info/$default_mtd/primaryboot)
		if [ $primaryboot -eq 0 ]; then
			echo 1 > /proc/boot_info/$default_mtd/primaryboot
		else
			echo 0 > /proc/boot_info/$default_mtd/primaryboot
		fi
	}

	emmcblock="$(find_mmc_part "$mtdname")"

	if [ -e "$emmcblock" ]; then
		do_flash_emmc $bin $emmcblock
	else
		do_flash_mtd $bin $mtdname
	fi

}

#senao dual image
change_active_partition() {
	active=$(setconfig -g 38)


    mkdir /var

	[ "0" = "${active}" ] && (
        echo "senao dual image: change active partition from $active to 1"
        setconfig -a 1
        setconfig -a 2 -s 38 -d 1
        setconfig -a 5
	)

	[ "1" = "${active}" ] && (
        echo "senao dual image: change active partition from $active to 0"
		setconfig -a 1
		setconfig -a 2 -s 38 -d 0
		setconfig -a 5
	)
}

do_flash_ubi() {
	local bin=$1
	local mtdname=$2
	local mtdpart
	local primaryboot

	#senao dual image
	local isSnDualImage=0
	if [ "$mtdname" == "rootfs" ]; then
		local isrootfsDualImage=$(grep "\"rootfs_1\"" /proc/mtd | awk -F: '{print $1}')
		if [ -n "$isrootfsDualImage" ]; then
			local isSnDualImage=1;
			mtdname="rootfs_1";
			echo "senao dual image upgrade, upgrade the alternative partion: $mtdname"
		fi
	fi
	if [ "$mtdname" == "0:WIFIFW" ]; then
		local isWIFIDualImage=$(grep "\"0:WIFIFW_1\"" /proc/mtd | awk -F: '{print $1}')
		if [ -n "$isWIFIDualImage" ]; then
			local isSnDualImage=1;
			mtdname="0:WIFIFW_1";
			echo "senao dual image upgrade, upgrade the alternative partion: $mtdname"
		fi
	fi
	if [ $isSnDualImage == "0" ]; then
		mtdpart=$(grep "\"${mtdname}\"" /proc/mtd | awk -F: '{print $1}')
		ubidetach -f -p /dev/${mtdpart}
	fi

	# Fail safe upgrade
	[ -f /proc/boot_info/$mtdname/upgradepartition ] && {
		primaryboot=$(cat /proc/boot_info/$mtdname/primaryboot)
		if [ $primaryboot -eq 0 ]; then
			echo 1 > /proc/boot_info/$mtdname/primaryboot
		else
			echo 0 > /proc/boot_info/$mtdname/primaryboot
		fi

		mtdname=$(cat /proc/boot_info/$mtdname/upgradepartition)
	}

	mtdpart=$(grep "\"${mtdname}\"" /proc/mtd | awk -F: '{print $1}')

	ubiformat /dev/${mtdpart} -y -f /tmp/${bin}.bin

    #senao dual image
	if [ -n "$isrootfsDualImage" ]; then
        change_active_partition;
    fi
}

do_flash_tz() {
	local sec=$1
	local mtdpart=$(grep "\"0:QSEE\"" /proc/mtd | awk -F: '{print $1}')
	local emmcblock="$(find_mmc_part "0:QSEE")"

	if [ -n "$mtdpart" -o -e "$emmcblock" ]; then
		do_flash_failsafe_partition ${sec} "0:QSEE"
	else
		do_flash_failsafe_partition ${sec} "0:TZ"
	fi
}

do_flash_ddr() {
	local sec=$1
	local mtdpart=$(grep "\"0:CDT\"" /proc/mtd | awk -F: '{print $1}')
	local emmcblock="$(find_mmc_part "0:CDT")"

	if [ -n "$mtdpart" -o -e "$emmcblock" ]; then
		do_flash_failsafe_partition ${sec} "0:CDT"
	else
		do_flash_failsafe_partition ${sec} "0:DDRPARAMS"
	fi
}

to_upper ()
{
	echo $1 | awk '{print toupper($0)}'
}
image_is_nand()
{
	local nand_part="$(find_mtd_part "ubi_rootfs")"
	[ -e "$nand_part" ] || return 1

}
flash_section() {
	local sec=$1

	local board=$(ipq806x_board_name)
	local version=$(hexdump -n 1 -e '"%1d"' /sys/firmware/devicetree/base/soc_version_major)

	# senao upgrade uboot by ver
	local isUpgradeUboot=$2
	if [ $isUpgradeUboot == "0" ]; then
		case "${sec}" in
			hlos*) switch_layout linux; image_is_nand && return || do_flash_failsafe_partition ${sec} "0:HLOS";;
			rootfs*) switch_layout linux; image_is_nand && return || do_flash_failsafe_partition ${sec} "rootfs";;
			wififw-*) switch_layout linux; do_flash_failsafe_partition ${sec} "0:WIFIFW";;
			wififw_ubi-*) switch_layout linux; do_flash_ubi ${sec} "0:WIFIFW";;
			wififw_v${version}-*) switch_layout linux; do_flash_failsafe_partition ${sec} "0:WIFIFW";;
			wififw_ubi_v${version}-*) switch_layout linux; do_flash_ubi ${sec} "0:WIFIFW";;
			fs*) switch_layout linux; do_flash_failsafe_partition ${sec} "rootfs";;
			ubi*) switch_layout linux; image_is_nand || return && do_flash_ubi ${sec} "rootfs";;
			*) echo "Section ${sec} ignored"; return 1;;
		esac
	else
		case "${sec}" in
			hlos*) switch_layout linux; image_is_nand && return || do_flash_failsafe_partition ${sec} "0:HLOS";;
			rootfs*) switch_layout linux; image_is_nand && return || do_flash_failsafe_partition ${sec} "rootfs";;
			wififw-*) switch_layout linux; do_flash_failsafe_partition ${sec} "0:WIFIFW";;
			wififw_ubi-*) switch_layout linux; do_flash_ubi ${sec} "0:WIFIFW";;
			wififw_v${version}-*) switch_layout linux; do_flash_failsafe_partition ${sec} "0:WIFIFW";;
			wififw_ubi_v${version}-*) switch_layout linux; do_flash_ubi ${sec} "0:WIFIFW";;
			fs*) switch_layout linux; do_flash_failsafe_partition ${sec} "rootfs";;
			ubi*) switch_layout linux; image_is_nand || return && do_flash_ubi ${sec} "rootfs";;
			sbl1*) switch_layout boot; do_flash_partition ${sec} "0:SBL1";;
			sbl2*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:SBL2";;
			sbl3*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:SBL3";;
			mibib*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:MIBIB";;
			dtb-$(to_upper $board)*) switch_layout boot; do_flash_partition ${sec} "0:DTB";;
			u-boot*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:APPSBL";;
			ddr-$(to_upper $board)*) switch_layout boot; do_flash_ddr ${sec};;
			ddr-${board}-*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:DDRCONFIG";;
			ssd*) switch_layout boot; do_flash_partition ${sec} "0:SSD";;
			tz*) switch_layout boot; do_flash_tz ${sec};;
			apdp*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:APDP";;
			devcfg*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:DEVCFG";;
			rpm*) switch_layout boot; do_flash_failsafe_partition ${sec} "0:RPM";;
			*) echo "Section ${sec} ignored"; return 1;;
		esac
	fi
	echo "Flashed ${sec}"
}

erase_emmc_config() {
	local mtdpart=$(cat /proc/mtd | grep rootfs)
	local emmcblock="$(find_mmc_part "rootfs_data")"
	if [ -z "$mtdpart" -a -e "$emmcblock" ]; then
		yes | mkfs.ext4 "$emmcblock"
	fi
}

senao_get_vender_id() {
	get_image "$@" | dd bs=2 count=1 skip=3 2>/dev/null | hexdump -v -n 2 -e '1/1 "%02x"'
}

senao_get_product_id() {
	get_image "$@" | dd bs=2 count=1 skip=5 2>/dev/null | hexdump -v -n 2 -e '1/1 "%02x"'
}

senao_image_header_check(){
	senao_magic_long="$(get_magic_long "$1")"
	senao_vendor_id="$(senao_get_vender_id "$1")"
	senao_product_id="$(senao_get_product_id "$1")"
	eval "rm -rf /tmp/fw_id.txt"
	echo "senao_vendor_id = $senao_vendor_id" > /tmp/fw_id.txt
	echo "senao_product_id = $senao_product_id" >> /tmp/fw_id.txt
	echo "senao_magic_long = $senao_magic_long" >> /tmp/fw_id.txt
	check_senao_image_header $senao_magic_long $senao_vendor_id $senao_product_id && return 0
	return 1
}

platform_pre_upgrade() {
	cp /sbin/upgraded /tmp
	# senao upgrade uboot by ver: enlarge timeout from 30 sec to 60 sec
	ubus -t 120 call system nandupgrade "{\"path\": \"$1\" }"
}

platform_check_image() {
	local board=$(ipq806x_board_name)

	local mandatory_nand="ubi"
	local mandatory_nor_emmc="hlos fs"
	local mandatory_nor="hlos"
	local mandatory_section_found=0
	local ddr_section="ddr"
	local optional="sb11 sbl2 u-boot ddr-${board} ssd tz rpm"
	local ignored="mibib bootconfig"

	image_is_FIT $1 || return 1

	image_has_mandatory_section $1 ${mandatory_nand} && {\
		mandatory_section_found=1
	}

	image_has_mandatory_section $1 ${mandatory_nor_emmc} && {\
		mandatory_section_found=1
	}

	image_has_mandatory_section $1 ${mandatory_nor} && {\
		mandatory_section_found=1
	}

	if [ $mandatory_section_found -eq 0 ]; then
		echo "Error: mandatory section(s) missing from \"$1\". Abort..."
		return 1
	fi

	image_has_mandatory_section $1 $ddr_section && {\
		image_contains $1 ddr-$board || {\
			image_contains $1 ddr-$(to_upper $board) || {\
			return 1
			}
		}
	}
	for sec in ${optional}; do
		image_contains $1 ${sec} || {\
			echo "Warning: optional section \"${sec}\" missing from \"$1\". Continue..."
		}
	done

	for sec in ${ignored}; do
		image_contains $1 ${sec} && {\
			echo "Warning: section \"${sec}\" will be ignored from \"$1\". Continue..."
		}
	done

	image_demux $1 || {\
		echo "Error: \"$1\" couldn't be extracted. Abort..."
		return 1
	}

	[ -f /tmp/hlos_version ] && rm -f /tmp/*_version
	dumpimage -c $1
	if [[ "$?" == 0 ]];then
		return $?
	fi
	echo "Rebooting the system"
	reboot
}

platform_version_upgrade() {
	local version_files="appsbl_version sbl_version tz_version hlos_version rpm_version"
	local sys="/sys/devices/system/qfprom/qfprom0/"
	local tmp="/tmp/"

	for file in $version_files; do
		[ -f "${tmp}${file}" ] && {
			echo "Updating "${sys}${file}" with `cat "${tmp}${file}"`"
			echo `cat "${tmp}${file}"` > "${sys}${file}"
			rm -f "${tmp}${file}"
		}
	done
}

platform_do_upgrade() {
	local board=$(ipq806x_board_name)

	# senao upgrade uboot by ver
	local isUpgradeUboot=0
	local newUbootVer
	local currentUbootVer=$(setconfig -g 24 | awk -F. '{print $1$2$3}')

	# verify some things exist before erasing
	if [ ! -e $1 ]; then
		echo "Error: Can't find $1 after switching to ramfs, aborting upgrade!"
		reboot
	fi

	for sec in $(print_sections $1); do
		if [ ! -e /tmp/${sec}.bin ]; then
			echo "Error: Cant' find ${sec} after switching to ramfs, aborting upgrade!"
			reboot
		fi

        # senao upgrade uboot by ver
        if [ ${sec%-*} == "u-boot" ]; then
            newUbootVer=$(cat /tmp/${sec}.bin | grep uboot_version | awk -F ':V| ' '{print $3}' | awk -F. '{print $1$2$3}')
            if [ $newUbootVer -gt $currentUbootVer ]; then
                isUpgradeUboot=1;
            fi
            echo "newUbootVer=$newUbootVer" > /dev/console
            echo "currentUbootVer=$currentUbootVer" > /dev/console
            echo "isUpgradeUboot=$isUpgradeUboot" > /dev/console
        fi
	done

	case "$board" in
	db149 | ap148 | ap145 | ap148_1xx | db149_1xx | db149_2xx | ap145_1xx | ap160 | ap160_2xx | ap161 | ak01_1xx | ap-dk01.1-c1 | ap-dk01.1-c2 | ap-dk04.1-c1 | ap-dk04.1-c2 | ap-dk04.1-c3 | ap-dk04.1-c4 | ap-dk04.1-c5 | ap-dk04.1-c6 | ap-dk05.1-c1 |  ap-dk06.1-c1 | ap-dk07.1-c1 | ap-dk07.1-c2 | ap-dk07.1-c3 | ap-dk07.1-c4 | ap-hk01-c1 | ap-hk01-c2 | ap-hk01-c3 | ap-hk01-c4 | ap-hk02 | ap-hk05 | ap-hk06 | ap-hk07 | ap-hk08 | ap-hk09 | ap-ac01.1 | ap-ac01.2 | db-hk01 | db-hk02)
		for sec in $(print_sections $1); do
			#senao upgrade uboot by ver
			flash_section ${sec} $isUpgradeUboot
		done

		switch_layout linux
		# update bootconfig to register that fw upgrade has been done
		do_flash_bootconfig bootconfig "0:BOOTCONFIG"
		do_flash_bootconfig bootconfig1 "0:BOOTCONFIG1"
		platform_version_upgrade

		erase_emmc_config
		return 0;
		;;
	esac

	echo "Upgrade failed!"
	return 1;
}

platform_copy_config() {
	local nand_part="$(find_mtd_part "ubi_rootfs")"
	local emmcblock="$(find_mmc_part "rootfs_data")"

	if [ -e "$nand_part" ]; then
		local mtdname=rootfs
		local mtdpart

		[ -f /proc/boot_info/$mtdname/upgradepartition ] && {
			mtdname=$(cat /proc/boot_info/$mtdname/upgradepartition)
		}

        #senao dual image
    	local isrootfsDualImage=$(grep "\"rootfs_1\"" /proc/mtd | awk -F: '{print $1}')
    	if [ -n "$isrootfsDualImage" ]; then
            if [ "$mtdname" == "rootfs" ]; then
                mtdname="rootfs_1";
                echo "senao dual image upgrade, save conf to the alternative partion: $mtdname"
            fi
        fi


		mtdpart=$(grep "\"${mtdname}\"" /proc/mtd | awk -F: '{print $1}')
		ubiattach -p /dev/${mtdpart}

        #senao dual image
    	if [ -n "$isrootfsDualImage" ]; then
		    mount -t ubifs ubi2:rootfs_data /tmp/overlay
        else
		    mount -t ubifs ubi0:rootfs_data /tmp/overlay
        fi

		cp /tmp/sysupgrade.tgz /tmp/overlay/
		sync
		umount /tmp/overlay
	elif [ -e "$emmcblock" ]; then
		mount -t ext4 "$emmcblock" /tmp/overlay
		cp /tmp/sysupgrade.tgz /tmp/overlay/
		sync
		umount /tmp/overlay
	fi
}

